/*
 * Copyright 2015-2020 The OpenZipkin Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

import { setupI18n } from '@lingui/core';

import enMessages from '../translations/en/messages';
import esMessages from '../translations/es/messages';
import zhCnMessages from '../translations/zh-cn/messages';

const localeStorageKey = 'localeOverride';

const catalogs = {
  en: enMessages,
  es: esMessages,
  'zh-cn': zhCnMessages,
};

export function getLocale(): string {
  const override = localStorage.getItem(localeStorageKey);
  if (override) {
    return override;
  }

  const browserLanguage = navigator.language.toLowerCase();

  if (Object.prototype.hasOwnProperty.call(catalogs, browserLanguage)) {
    return browserLanguage;
  }

  const hyphenIndex = browserLanguage.indexOf('-');
  if (hyphenIndex >= 0) {
    const stripped = browserLanguage.substring(0, hyphenIndex);
    if (Object.prototype.hasOwnProperty.call(catalogs, stripped)) {
      return stripped;
    }
  }

  // Default to English if we don't support the current browser language.
  return 'en';
}

export function setLocale(locale: string) {
  localStorage.setItem(localeStorageKey, locale);
}

export const i18n = setupI18n({
  catalogs: catalogs as any,
  locale: getLocale(),
});
